@@ -3,17 +3,18 @@ import logging
 from collections.abc import MutableMapping
 from collections.abc import Sequence
 from typing import Callable
+from typing import Iterable
 from typing import List
 from typing import Type
 
 import numpy as np
 from sklearn.base import BaseEstimator
-from sklearn.pipeline import FeatureUnion
 
 from sklearn_instrumentation.config import DEFAULT_EXCLUDE
 from sklearn_instrumentation.config import DEFAULT_METHODS
 from sklearn_instrumentation.utils import get_delegator
 from sklearn_instrumentation.utils import get_estimators_in_package
+from sklearn_instrumentation.utils import get_method_class
 from sklearn_instrumentation.utils import is_delegator
 from sklearn_instrumentation.utils import method_is_inherited
 
@@ -97,6 +98,16 @@ class SklearnInstrumentor:
             return wrapper
 
 
+    By default, classes derived from ``sklearn.tree._classes.BaseDecisionTree`` are
+    excluded from instrumentation.
+
+    By default, methods on which instrumentation is applied includes
+    ``_fit``, ``_predict``, ``_predict_proba``, ``_transform``, ``fit``, ``predict``,
+    ``predict_proba``, and ``transform``.
+
+    Methods which are properties are **not** instrumented on instances, but **are**
+    instrumented on classes.
+
     :param Callable instrument: A decorator to apply to sklearn estimator methods. The
         wrapping function signature should be ``(func, **dkwargs)``, where ``func`` is
         the target method and ``dkwargs`` is the instrument_kwargs argument of the
@@ -119,6 +130,10 @@ class SklearnInstrumentor:
         self.methods = methods or DEFAULT_METHODS
         self.exclude = tuple(exclude or DEFAULT_EXCLUDE)
 
+    @classmethod
+    def _get_instrumentation_attribute_name(cls, method_name: str):
+        return f"_skli_{method_name}"
+
     # region instrumentation estimator
     def instrument_estimator(
         self,
@@ -154,8 +169,6 @@ class SklearnInstrumentor:
             )
 
     def _instrument_recursively(self, obj: object, instrument_kwargs=None):
-        if isinstance(obj, FeatureUnion):
-            pass
         if isinstance(obj, (*self.exclude, str, np.ndarray)):
             return
 
@@ -180,10 +193,10 @@ class SklearnInstrumentor:
         method_name: str,
         instrument_kwargs: dict = None,
     ):
-        class_attribute = getattr(type(estimator), method_name, None)
+        class_attribute = getattr(estimator.__class__, method_name, None)
         if isinstance(class_attribute, property):
             logger.debug(
-                f"Not instrumenting property: {estimator.__class__.__qualname__}.{method_name}",
+                f"Not instrumenting property on instance of: {estimator.__class__.__qualname__}.{method_name}",
             )
             return
 
@@ -193,10 +206,21 @@ class SklearnInstrumentor:
 
         dkwargs = instrument_kwargs or self.instrument_kwargs
 
-        instr = getattr(estimator, f"_skli_{method_name}", None)
+        instr_attrib_name = self._get_instrumentation_attribute_name(
+            method_name=method_name
+        )
+        instr = getattr(
+            estimator,
+            instr_attrib_name,
+            None,
+        )
         if instr is None:
             instr = SklearnMethodInstrumentation(method)
-            setattr(estimator, f"_skli_{method_name}", instr)
+            setattr(
+                estimator,
+                instr_attrib_name,
+                instr,
+            )
 
         instr.add(instrument=self.instrument, instrument_kwargs=dkwargs)
         setattr(
@@ -285,6 +309,88 @@ class SklearnInstrumentor:
 
     # endregion
 
+    # region estimator classes
+    def instrument_estimators_classes(self, estimators: Iterable[BaseEstimator]):
+        """Instrument the classes (not the instances) found in the estimators.
+
+        Lighter version of ``instrument_package``. Instead of crawling the package
+        modules and dynamically importing objects, crawls the estimators' hierarchies
+        and only instruments classes which are already imported. This is generally
+        faster and uses less memory.
+
+        Inspects the (meta)estimator hierarchy, only instrumenting the classes found
+        in the estimator, its methods, and its attributes.
+
+        :param Iterable[BaseEstimator] estimators: Several estimator instances of which
+            to instrument related classes.
+        """
+        classes = set()
+        for estimator in estimators:
+            classes = classes.union(self._get_estimator_classes(estimator))
+            self.instrument_classes(estimators=classes)
+
+    def instrument_estimator_classes(self, estimator: BaseEstimator):
+        """Instrument the classes (not the instances) found in the estimator.
+
+        Lighter version of ``instrument_package``. Instead of crawling the package
+        modules and dynamically importing objects, crawls the estimator hierarchy
+        and only instruments classes which are already imported. This is generally
+        faster and uses less memory.
+
+        Inspects the (meta)estimator hierarchy, only instrumenting the classes found
+        in the estimator, its methods, and its attributes.
+
+        :param BaseEstimator estimator: An estimator instance with which to instrument
+            related classes
+        """
+        classes = self._get_estimator_classes(estimator)
+        self.instrument_classes(estimators=classes)
+
+    def uninstrument_estimator_classes(
+        self, estimator: BaseEstimator, full: bool = False
+    ):
+        """Uninstrument the classes (not the instances) found in the estimator.
+
+        Inspects the (meta)estimator hierarchy, only uninstrumenting the classes found
+        in the estimator, its methods, and its attributes.
+
+        :param BaseEstimator estimator: An estimator instance with which to uninstrument
+            related classes
+        :param bool full: Whether to fully uninstrument the estimator classes.
+        """
+        classes = self._get_estimator_classes(estimator)
+        self.uninstrument_classes(estimators=classes, full=full)
+
+    def _get_estimator_classes(self, obj):
+        classes = set()
+        if isinstance(obj, (*self.exclude, str, np.ndarray)):
+            return classes
+
+        if isinstance(obj, BaseEstimator):
+            class_ = obj.__class__
+            classes.add(class_)
+            for k in dir(class_):
+                v = getattr(class_, k, None)
+                if v is None or not callable(v):
+                    continue
+                v = getattr(obj, k, None)
+                if hasattr(v, "__self__") and k in self.methods:
+                    classes.add(get_method_class(class_, k))
+
+        if hasattr(obj, "__dict__"):
+            for v in obj.__dict__.values():
+                classes = classes.union(self._get_estimator_classes(v))
+        elif isinstance(obj, MutableMapping):
+            for v in obj.values():
+                classes = classes.union(self._get_estimator_classes(v))
+        elif isinstance(obj, Sequence):
+            for o in obj:
+                classes = classes.union(self._get_estimator_classes(o))
+
+        return classes
+
+    # endregion
+
     # region instrumentation package
     def instrument_packages(self, package_names: List[str]):
         """Instrument multiple packages.
@@ -304,6 +410,17 @@ class SklearnInstrumentor:
         :param str package_name: A list of package names.
         """
         estimators = get_estimators_in_package(package_name=package_name)
+        self.instrument_classes(estimators=estimators)
+
+    def instrument_classes(self, estimators: Iterable[Type[BaseEstimator]]):
+        """Instrument multiple BaseEstimator classes.
+
+        Apply this instrumentor's decorator to the methods of several BaseEstimator
+        classes.
+
+        :param Iterable[Type[BaseEstimator]] estimators: Iterable of classes on which to
+            apply instrumentation.
+        """
         for estimator in estimators:
             self.instrument_class(estimator=estimator)
 
@@ -330,23 +447,33 @@ class SklearnInstrumentor:
         if class_method is None:
             return
 
-        if isinstance(class_method, property):
-            logger.debug(
-                f"Not instrumenting property: {estimator.__qualname__}.{method_name}",
-            )
+        if method_is_inherited(estimator=estimator, method_name=method_name):
             return
 
-        if method_is_inherited(estimator=estimator, method_name=method_name):
+        if isinstance(class_method, property):
+            self._instrument_property(estimator=estimator, method_name=method_name)
             return
 
         if is_delegator(func=class_method):
             self._instrument_delegator(delegator=class_method, method_name=method_name)
             return
 
-        instr = getattr(estimator, f"_skli_{method_name}", None)
+        instr_attrib_name = self._get_instrumentation_attribute_name(
+            method_name=method_name
+        )
+        instr = getattr(
+            estimator,
+            instr_attrib_name,
+            None,
+        )
         if instr is None:
             instr = SklearnMethodInstrumentation(class_method)
-            setattr(estimator, f"_skli_{method_name}", instr)
+            setattr(
+                estimator,
+                instr_attrib_name,
+                instr,
+            )
+
         instr.add(instrument=self.instrument, instrument_kwargs=self.instrument_kwargs)
         setattr(
             estimator,
@@ -354,8 +481,25 @@ class SklearnInstrumentor:
             instr.wrapper,
         )
 
+    def _instrument_property(self, estimator: Type[BaseEstimator], method_name: str):
+        instr_attrib_name = self._get_instrumentation_attribute_name(
+            method_name=method_name
+        )
+        property_: property = getattr(estimator, method_name)
+        wrapped_method = property_.fget
+        instr = getattr(estimator, instr_attrib_name, None)
+        if instr is None:
+            instr = SklearnMethodInstrumentation(wrapped_method)
+            setattr(estimator, instr_attrib_name, instr)
+
+        instr.add(instrument=self.instrument, instrument_kwargs=self.instrument_kwargs)
+
+        setattr(estimator, method_name, property(instr.wrapper))
+
     def _instrument_delegator(self, delegator: Callable, method_name: str):
-        instr_attrib_name = self._get_instrumentation_attribute_name(method_name)
+        instr_attrib_name = self._get_instrumentation_attribute_name(
+            method_name=method_name
+        )
         descriptor = get_delegator(delegator)
         instr = getattr(descriptor, instr_attrib_name, None)
         if instr is None:
@@ -393,15 +537,31 @@ class SklearnInstrumentor:
         :param bool full: Whether to fully uninstrument the packages.
         """
         estimators = get_estimators_in_package(package_name=package_name)
+        self.uninstrument_classes(estimators=estimators, full=full)
+
+    def uninstrument_classes(
+        self, estimators: Iterable[Type[BaseEstimator]], full: bool = False
+    ):
+        """Uninstrument BaseEstimator classes.
+
+        Remove this instrumentor's decorator from the methods of a BaseEstimator
+        classes.
+
+        :param Iterable[Type[BaseEstimator]] estimators: Classes from which to remove
+            instrumentation.
+        :param bool full: Whether to fully uninstrument the estimator classes.
+        """
         for estimator in estimators:
             self.uninstrument_class(estimator=estimator, full=full)
 
     def uninstrument_class(self, estimator: Type[BaseEstimator], full: bool = False):
-        """Instrument a BaseEstimator class.
+        """Uninstrument a BaseEstimator class.
 
-        Apply this instrumentor's decorator to the methods of a BaseEstimator class.
+        Remove this instrumentor's decorator to the methods of a BaseEstimator class.
 
-        :param Type[BaseEstimator] estimator: A class on which to apply instrumentation.
+        :param Type[BaseEstimator] estimator: A class from which to remove
+            instrumentation.
+        :param bool full: Whether to fully uninstrument the estimator class.
         """
         for method_name in self.methods:
             self._uninstrument_class_method(
@@ -418,10 +578,11 @@ class SklearnInstrumentor:
         if class_method is None:
             return
 
-        if isinstance(class_method, property):
+        if method_is_inherited(estimator=estimator, method_name=method_name):
             return
 
-        if method_is_inherited(estimator=estimator, method_name=method_name):
+        if isinstance(class_method, property):
+            self._uninstrument_property(estimator=estimator, method_name=method_name)
             return
 
         if is_delegator(func=class_method):
@@ -445,6 +606,27 @@ class SklearnInstrumentor:
         else:
             setattr(estimator, method_name, instr.wrapper)
 
+    def _uninstrument_property(
+        self, estimator: Type[BaseEstimator], method_name: str, full: bool = False
+    ):
+        instr_attrib_name = self._get_instrumentation_attribute_name(method_name)
+        instr = getattr(estimator, instr_attrib_name, None)
+        if instr is None:
+            return
+
+        instr: SklearnMethodInstrumentation
+        instr.remove(instrument=self.instrument)
+
+        if full or instr.empty():
+            setattr(
+                estimator,
+                method_name,
+                property(instr.wrapped),
+            )
+            delattr(estimator, instr_attrib_name)
+        else:
+            setattr(estimator, method_name, property(instr.wrapper))
+
     def _uninstrument_delegator(
         self, delegator: Callable, method_name: str, full: bool = False
     ):
@@ -467,7 +649,4 @@ class SklearnInstrumentor:
         else:
             setattr(descriptor, "fn", instr.wrapper)
 
-    def _get_instrumentation_attribute_name(self, method_name: str):
-        return f"_skli_{method_name}"
-
     # endregion
