@@ -1,6 +1,6 @@
 from abc import ABC
 from abc import abstractmethod
-from collections import Callable
+from collections.abc import Callable
 from functools import wraps
 
 
@@ -8,14 +8,14 @@ class BaseInstrument(ABC):
     """Base class for instruments."""
 
     @abstractmethod
-    def __call__(self, func: Callable, **dkwargs):
+    def __call__(self, func: Callable, **dkwargs):  # pragma: no cover
         pass
 
 
 class Identity(BaseInstrument):
     """Identity instrument which decorates with a no-op."""
 
-    def __call__(self, func: Callable, **dkwargs):
+    def __call__(self, func: Callable, **dkwargs):  # pragma: no cover
         @wraps(func)
         def wrapper(*args, **kwargs):
             return func(*args, **kwargs)
