@@ -1,4 +1,5 @@
 from typing import Callable
+from typing import Iterable
 from typing import List
 from typing import MutableMapping
 from typing import Sequence
@@ -7,6 +8,7 @@ from typing import Type
 import numpy as np
 from sklearn.base import BaseEstimator
 
+from sklearn_instrumentation.instrumentor import SklearnInstrumentor
 from sklearn_instrumentation.instrumentor import SklearnMethodInstrumentation
 from sklearn_instrumentation.utils import get_delegator
 from sklearn_instrumentation.utils import get_estimators_in_package
@@ -63,8 +65,11 @@ class SklearnInstrumentionAsserter:
         if isinstance(class_attribute, property):
             return
 
+        instr_attrib_name = SklearnInstrumentor._get_instrumentation_attribute_name(
+            method_name=method_name
+        )
         instr: SklearnMethodInstrumentation = getattr(
-            estimator, f"_skli_{method_name}", None
+            estimator, instr_attrib_name, None
         )
         if hasattr(estimator, method_name):
             assert instr is not None
@@ -136,6 +141,9 @@ class SklearnInstrumentionAsserter:
 
     def assert_instrumented_package(self, package_name: str = "sklearn"):
         estimators = get_estimators_in_package(package_name=package_name)
+        self.assert_instrumented_classes(estimators=estimators)
+
+    def assert_instrumented_classes(self, estimators: Iterable[Type[BaseEstimator]]):
         for estimator in estimators:
             self.assert_instrumented_class(estimator=estimator)
 
@@ -154,10 +162,13 @@ class SklearnInstrumentionAsserter:
         if class_method is None:
             return
 
-        if isinstance(class_method, property):
+        if method_is_inherited(estimator=estimator, method_name=method_name):
             return
 
-        if method_is_inherited(estimator=estimator, method_name=method_name):
+        if isinstance(class_method, property):
+            self._assert_instrumented_property(
+                estimator=estimator, method_name=method_name
+            )
             return
 
         if is_delegator(func=class_method):
@@ -173,8 +184,23 @@ class SklearnInstrumentionAsserter:
         else:
             assert instr is None
 
+    def _assert_instrumented_property(
+        self, estimator: Type[BaseEstimator], method_name: str
+    ):
+        instr_attrib_name = SklearnInstrumentor._get_instrumentation_attribute_name(
+            method_name=method_name
+        )
+        property_: property = getattr(estimator, method_name)
+        instr: SklearnMethodInstrumentation = getattr(estimator, instr_attrib_name)
+
+        assert self.instrumentor.instrument in instr.instruments
+        assert self.instrumentor.instrument_kwargs in instr.instrument_kwargs
+        assert instr.wrapper == property_.fget
+
     def _assert_instrumented_delegator(self, delegator: Callable, method_name: str):
-        instr_attrib_name = f"_skli_{method_name}"
+        instr_attrib_name = SklearnInstrumentor._get_instrumentation_attribute_name(
+            method_name=method_name
+        )
         descriptor = get_delegator(delegator)
         instr: SklearnMethodInstrumentation = getattr(
             descriptor, instr_attrib_name, None
@@ -192,6 +218,11 @@ class SklearnInstrumentionAsserter:
 
     def assert_uninstrumented_package(self, package_name: str, full: bool = False):
         estimators = get_estimators_in_package(package_name=package_name)
+        self.assert_uninstrumented_classes(estimators=estimators, full=full)
+
+    def assert_uninstrumented_classes(
+        self, estimators: Iterable[Type[BaseEstimator]], full: bool = False
+    ):
         for estimator in estimators:
             self.assert_uninstrumented_class(estimator=estimator, full=full)
 
@@ -212,10 +243,13 @@ class SklearnInstrumentionAsserter:
         if class_method is None:
             return
 
-        if isinstance(class_method, property):
+        if method_is_inherited(estimator=estimator, method_name=method_name):
             return
 
-        if method_is_inherited(estimator=estimator, method_name=method_name):
+        if isinstance(class_method, property):
+            self._assert_uninstrumented_property(
+                estimator=estimator, method_name=method_name, full=full
+            )
             return
 
         if is_delegator(func=class_method):
@@ -231,14 +265,15 @@ class SklearnInstrumentionAsserter:
         if instr is not None:
             assert self.instrumentor.instrument not in instr.instruments
 
-    def _assert_uninstrumented_delegator(
-        self, delegator: Callable, method_name: str, full: bool = False
+    def _assert_uninstrumented_property(
+        self, estimator: Type[BaseEstimator], method_name: str, full: bool = False
     ):
-        instr_attrib_name = f"_skli_{method_name}"
+        instr_attrib_name = SklearnInstrumentor._get_instrumentation_attribute_name(
+            method_name=method_name
+        )
         instr: SklearnMethodInstrumentation = getattr(
-            delegator, instr_attrib_name, None
+            estimator, instr_attrib_name, None
         )
-        descriptor = get_delegator(delegator)
 
         if full:
             assert instr is None
@@ -246,4 +281,24 @@ class SklearnInstrumentionAsserter:
         if instr is not None:
             assert self.instrumentor.instrument not in instr.instruments
             assert self.instrumentor.instrument_kwargs not in instr.instrument_kwargs
+            property_: property = getattr(estimator, method_name)
+            assert instr.wrapped == property_.fget
+
+    def _assert_uninstrumented_delegator(
+        self, delegator: Callable, method_name: str, full: bool = False
+    ):
+        instr_attrib_name = SklearnInstrumentor._get_instrumentation_attribute_name(
+            method_name=method_name
+        )
+        instr: SklearnMethodInstrumentation = getattr(
+            delegator, instr_attrib_name, None
+        )
+
+        if full:
+            assert instr is None
+
+        if instr is not None:
+            assert self.instrumentor.instrument not in instr.instruments
+            assert self.instrumentor.instrument_kwargs not in instr.instrument_kwargs
+            descriptor = get_delegator(delegator)
             assert instr.wrapped == descriptor.fn
